From 692ed3e237fc645253fff3fbb932ad45a525a7ba Mon Sep 17 00:00:00 2001
From: Marco Genasci <fedeliallalinea@gmail.com>
Date: Wed, 12 Jan 2022 16:08:41 +0100
Subject: [PATCH 1/2] [X11] Fix tag dragging in Mutter and KWin

ref: https://bugs.chromium.org/p/chromium/issues/detail?id=1279532
caused by: https://bugs.chromium.org/p/chromium/issues/detail?id=1263023
Patches from:
https://chromium-review.googlesource.com/c/chromium/src/+/3373204
https://chromium-review.googlesource.com/c/chromium/src/+/3355376
---
 .../x11/x11_topmost_window_finder.cc            | 17 +++++------------
 1 file changed, 5 insertions(+), 12 deletions(-)

diff --git a/ui/platform_window/x11/x11_topmost_window_finder.cc b/ui/platform_window/x11/x11_topmost_window_finder.cc
index 50e75e17f..e20bf0abf 100644
--- a/ui/platform_window/x11/x11_topmost_window_finder.cc
+++ b/ui/platform_window/x11/x11_topmost_window_finder.cc
@@ -44,15 +44,8 @@ bool EnumerateChildren(ShouldStopIteratingCallback should_stop_iterating,
   for (iter = windows.rbegin(); iter != windows.rend(); iter++) {
     if (IsWindowNamed(*iter) && should_stop_iterating.Run(*iter))
       return true;
-  }
-
-  // If we're at this point, we didn't find the window we're looking for at the
-  // current level, so we need to recurse to the next level.  We use a second
-  // loop because the recursion and call to XQueryTree are expensive and is only
-  // needed for a small number of cases.
-  if (++depth <= max_depth) {
-    for (iter = windows.rbegin(); iter != windows.rend(); iter++) {
-      if (EnumerateChildren(should_stop_iterating, *iter, max_depth, depth))
+    if (depth < max_depth) {
+      if (EnumerateChildren(should_stop_iterating, *iter, max_depth, depth + 1))
         return true;
     }
   }
@@ -68,9 +61,9 @@ bool EnumerateAllWindows(ShouldStopIteratingCallback should_stop_iterating,
 
 void EnumerateTopLevelWindows(
     ui::ShouldStopIteratingCallback should_stop_iterating) {
-  // Some WMs parent 'top-level' windows in unnamed actual top-level windows
-  // (ion WM), so extend the search depth to all children of top-level windows.
-  const int kMaxSearchDepth = 1;
+  // WMs may reparent toplevel windows inside their own containers, so extend
+  // the search to all grandchildren of all toplevel windows.
+  const int kMaxSearchDepth = 2;
   ui::EnumerateAllWindows(should_stop_iterating, kMaxSearchDepth);
 }
 
-- 
2.34.1

